﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ProbabilityDistribution1
{
    public partial class Exercise : Form
    {
        List<Distribution> values;

        public Exercise()
        {
            InitializeComponent();
        }

        private void Exercise_Load(object sender, EventArgs e)
        {
            values = new List<Distribution>();
        }

        void ShowValues()
        {
            lvwValues.Items.Clear();

            foreach (Distribution dist in values)
            {
                ListViewItem lviValue = new ListViewItem(dist.X.ToString());
                lviValue.SubItems.Add(dist.PofX.ToString());
                lviValue.SubItems.Add(dist.X.ToString() + " * " +
                                      dist.PofX.ToString() + " = " +
                                      (dist.X * dist.PofX).ToString());
                lvwValues.Items.Add(lviValue);
            }

            txtCount.Text = values.Count.ToString();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            int x = 0;
            double p = 0.00, sum = 0.00;
            Distribution dist = null;

            // Check that the user entered a value for x
            if (txtX.Text.Length == 0)
            {
                MessageBox.Show("You must enter the x value.",
                                "Probability Distribution");
                return;
            }

            // Test that the user entered a value for P(x)
            if (txtPofX.Text.Length == 0)
            {
                MessageBox.Show("You must enter the P(x) value.",
                                "Probability Distribution");
                return;
            }

            // Get the value for x
            try
            {
                x = int.Parse(txtX.Text);
            }
            catch (FormatException)
            {
                MessageBox.Show("The value you entered is invalid.",
                                "Probability Distribution");
            }

            // Get the value for P(x)
            try
            {
                p = double.Parse(txtPofX.Text);
            }
            catch (FormatException)
            {
                MessageBox.Show("The value you entered is invalid.",
                                "Probability Distribution");
            }

            // Create a Distribution value
            dist = new Distribution(x, p);
            // Add the value to the list
            values.Add(dist);

            ShowValues();

            // Calculate the sum of the P(x) values
            foreach (Distribution d in values)
                sum += d.PofX;
            txtSum.Text = sum.ToString("F");

            txtX.Text = "";
            txtPofX.Text = "";
            txtX.Focus();

            // Test the first requirement
            if (sum != 1) // The first rule is not respected
            {
                MessageBox.Show("The first rule is not respected",
                                "Probability Distribution");
                return;
            }

            // Test the second requirement
            foreach (Distribution d in values)
            {
                if ((d.PofX < 0.00) || (d.PofX > 1)) // The second rule is not respected
                {
                    MessageBox.Show("The second rule is not respected",
                                    "Probability Distribution");
                    return;
                }
            }
        }

        private void btnCalculate_Click(object sender, EventArgs e)
        {
            double mean = 0.00, variance = 0.00;
            double sum = 0.00, stddev = 0.00;

            foreach (Distribution dist in values)
                mean += (dist.X * dist.PofX);

            lvwValues.Items.Clear();

            foreach (Distribution dist in values)
            {
                ListViewItem lviValue = new ListViewItem(dist.X.ToString());
                lviValue.SubItems.Add(dist.PofX.ToString());

                lviValue.SubItems.Add(dist.X.ToString() + " * " +
                                      dist.PofX.ToString() + " = " +
                                      (dist.X * dist.PofX).ToString());
                lviValue.SubItems.Add("(" + dist.X.ToString() + " - " +
                                      mean.ToString() + ")² * " +
                                      dist.PofX.ToString() + " = " +
                                      Math.Pow(dist.X - mean, 2) * dist.PofX);
                lvwValues.Items.Add(lviValue);
            }

            foreach (Distribution dist in values)
                variance += Math.Pow(dist.X - mean, 2) * dist.PofX;

            foreach (Distribution dist in values)
                sum += Math.Pow(dist.X, 2) * dist.PofX;

            stddev = Math.Sqrt(sum - Math.Pow(mean, 2));

            txtMean.Text = mean.ToString();
            txtVariance.Text = variance.ToString();
            txtStdDev.Text = stddev.ToString();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            txtX.Text = "";
            txtPofX.Text = "";
            lvwValues.Items.Clear();
            txtCount.Text = "";
            txtSum.Text = "";
            txtMean.Text = "";
            txtVariance.Text = "";
            txtStdDev.Text = "";
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }
    }

    public class Distribution
    {
        public int X { get; set; }
        public double PofX { get; set; }

        public Distribution(int x, double p)
        {
            X = x;
            PofX = p;
        }
    }
}
